#include <nds.h>
#include <string.h>
#include "basics.h"
#include "persistent.h"
#include "global.h"



/* Pour viter d'allouer un gros truc sur la pile. */
static struct save_data tmp_data ;



static void bytes_memcpy (u8* dest, u8 const *src, u32 size)
{
  while (size--) *dest++ = *src++ ;
}


void save_scores ()
{
  u16 i ;

  /* It is now safe to write GBA Cartridge memory. */
  WAIT_CR &= ~0x80 ;

  tmp_data.magic_tag = MAGICNUMBER ;

  /* On recopie les scores en mmoire temporaire. */
  for (i = 0; i < NB_HIGHSCORES; i++) {
    tmp_data.highscores.standard[i] = global_highscores.standard[i] ;
    tmp_data.highscores.arcade[i] = global_highscores.arcade[i] ;
  }

  bytes_memcpy ((u8*) SRAM, (u8 const*) &tmp_data, sizeof (struct save_data)) ;
}




boolean read_scores ()
{
  u16 i ;

  /* It is now safe to read GBA Cartridge memory. */
  WAIT_CR &= ~0x80 ;
  bytes_memcpy ((u8*) &tmp_data, (u8 const*) SRAM, sizeof (struct save_data)) ;

  if (tmp_data.magic_tag != MAGICNUMBER) return (FALSE) ;

  for (i = 0; i < NB_HIGHSCORES; i++) {
    global_highscores.standard[i] = tmp_data.highscores.standard[i] ;
    global_highscores.arcade[i] = tmp_data.highscores.arcade[i] ;
  }

  return (TRUE) ;
}



boolean check_if_highscore (enum game_type gtype, u32 score)
{
  short i, j ;
  u32 *score_table = NULL ;

  switch (gtype) {
    case Game_standard:
      score_table = &(global_highscores.standard[0]) ;
      break ;
    case Game_arcade:
      score_table = &(global_highscores.arcade[0]) ;
      break ;
  }

  if (score_table == NULL) return (FALSE) ;
  /* Recherche d'un point d'entre dans la table des highscores. */
  for (i = 0; i < NB_HIGHSCORES; i++) {
    if (score > score_table[i]) {
      /* Dcalage des scores infrieurs. */
      for (j = (NB_HIGHSCORES - 1); j > i; j--)
	score_table[j] = score_table[j - 1] ;
      score_table[i] = score ;
      return (TRUE) ;
    }
  }    /* Fin de for (i = 0; i < NB_HIGHSCORES; i++). */

  return (FALSE) ;
}
